﻿using System;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using System.IO;
using Moonfish.Core;

public class AssetImporter
{
    public static void SaveTag(string tagClass, string tagName, TagBlock tagBlock)
    {
        tagName = tagName.Replace('\\', '/') + "." + tagClass;

        RecursiveCreateDirectory(Path.GetDirectoryName(tagName));

        switch (tagClass)
        { 
            case "mode":
                SaveMeshes(tagName, (model)tagBlock);
                break;
            default:
                SavePrefab(tagClass, tagName, tagBlock);
                break;
        }
    }

    private static void SavePrefab(string tagClass, string tagName, TagBlock tagBlock)
    {
        string prefabName = "Assets/Tags/" + tagName + ".prefab";
        
        GameObject gameObject = new GameObject(Path.GetFileNameWithoutExtension(tagName));
        TagComponent tc = (TagComponent)gameObject.AddComponent(tagClass);
        tc.TagBlock = tagBlock;
        tc.Import();

        PrefabUtility.CreatePrefab(prefabName, gameObject, ReplacePrefabOptions.ConnectToPrefab);
    }

    private static void SaveMesh(string tagName, Moonfish.Core.Model.Mesh mesh)
    {
        string assetName = "Assets/Tags/" + tagName + ".asset";

        AssetDatabase.CreateAsset(MeshImporter.Import(mesh), assetName);
        AssetDatabase.SaveAssets();
    }

    private static void SaveMeshes(string tagName, Moonfish.Core.model tag)
    {
        Moonfish.Core.Model.Mesh mesh = new Moonfish.Core.Model.Mesh();
        for (int r = 0; r < tag.Regions.Count; r++)
        {
            BinaryReader br = new BinaryReader(tag.Regions[r].Permutations[2].GetMemory());
            br.BaseStream.Position = 14;
            int index = br.ReadInt16();
            br.Close();

            mesh.Load(tag.Sections[index].Raw, tag.Sections[index].Resources, tag.Compression[0]);

            AssetImporter.SaveMesh(tagName + "-" + index, mesh);
        }
    }

    private static void RecursiveCreateDirectory(string directory)
    {
        string[] split = directory.Split('/');
        directory = "";

        for (int i = 0; i < split.Length; i++)
        {
            string nextFolder = directory + "/" + split[i];
            if (!Directory.Exists(Application.dataPath + "/Tags" + nextFolder))
                AssetDatabase.CreateFolder("Assets/Tags" + directory, split[i]);
            directory = nextFolder;
        }
    }
}
