using UnityEngine;
using UnityEditor;
using System.Collections;
using System.IO;
using System;
using H2.Streams;
using Moonfish.Core;

public class MapWizard : ScriptableWizard
{
    [UnityEditor.MenuItem("Halo 2/Open Map")]
    static void CreateWizard()
    {
        string filename = EditorUtility.OpenFilePanel("Choose Map File", Application.dataPath, "map");
        if (!File.Exists(filename)) return;

        map = new H2.Streams.MapStream(filename, null);

        classes = map.Tags.GetStringArrayOfClasses();
        tags = new string[classes.Length][];
        foldoutsStates = new bool[classes.Length];
        Array.Sort<string>(classes);

        for (int c = 0; c < classes.Length; c++)
        {
            for (int t = 0; t < classes.Length; t++)
                tags[c] = map.Tags.GetStringArrayOfClass(classes[c]);
        }

        map.Close();

        ScriptableWizard.DisplayWizard<MapWizard>(filename, "Import");
    }

    static H2.Streams.MapStream map;
    static string[] classes;
    static string[][] tags;

    static Vector2 scrollPosition;
    static bool[] foldoutsStates;

    void OnGUI()
    {
        scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition, GUIStyle.none);

        for (int i = 0; i < classes.Length; i++)
        {
            if (classes[i] != "mode") continue;
            foldoutsStates[i] = ShowTagList(classes[i], tags[i], foldoutsStates[i]);
        }

        EditorGUILayout.EndScrollView();
    }

    bool ShowTagList(string tagClass, string[] tags, bool isOpen)
    {
        if (EditorGUILayout.Foldout(isOpen, tagClass))
        {
            for (int i = 0; i < tags.Length; i++)
            {
                EditorGUILayout.BeginHorizontal();
                GUILayout.Space(10.0f);
                EditorGUILayout.LabelField(tags[i]);
                if (GUILayout.Button("Import")) Import(tagClass, tags[i]);
                EditorGUILayout.EndHorizontal();
            }
            return true;
        }
        return false;
    }

    private void Import(string tagClass, string tagName)
    {
        Moonfish.Core.MapStream mcmap = new Moonfish.Core.MapStream(map.FilePath);
        TagBlock tagBlock = mcmap[tagClass, tagName].Export();
        mcmap.Close();

        AssetImporter.SaveTag(tagClass, tagName, tagBlock);
    }
}
