using UnityEngine;
using UnityEditor;
using System.Collections;
using System.IO;
using Moonfish.Core;

public class ModelImporterWizard : ScriptableWizard
{
    [MenuItem("Halo 2/Import/Model")]
    static void CreateWizard()
    {
        ScriptableWizard.DisplayWizard<ModelImporterWizard>("Select Model To Import", "Import");
    }

    void OnWizardUpdate()
    {
    }

    void OnWizardCreate()
    {
        string assetName = "Assets/Tags/" + "objects/warthog/warthog.mode" + ".asset";
        Mesh mesh = (Mesh)AssetDatabase.LoadAssetAtPath(assetName, typeof(Mesh));
        //if (mesh != null) return;

        Debug.Log("Importing Mesh...");
        mesh = Import();
        if (!Directory.Exists(Application.dataPath + "/Tags")) AssetDatabase.CreateFolder("Assets", "Tags");
        if (!Directory.Exists(Application.dataPath + "/Tags/objects")) AssetDatabase.CreateFolder("Assets/Tags", "objects");
        if (!Directory.Exists(Application.dataPath + "/Tags/objects/warthog")) AssetDatabase.CreateFolder("Assets/Tags/objects", "warthog");
        AssetDatabase.CreateAsset(mesh, assetName);
        AssetDatabase.SaveAssets();
        Debug.Log("Saving Mesh...");

        //*********************************
        //Import Meta
        GameObject gameObject = new GameObject("Warthog");
        //Vechicle v = gameObject.AddComponent<Vechicle>();
        //v.Import();
        PrefabUtility.CreatePrefab("Assets/Tags/objects/warthog/warthog.vehi.prefab", gameObject, ReplacePrefabOptions.ConnectToPrefab);
    }

    public Mesh Import()
    {


        Mesh mesh = new Mesh();
        mesh.Clear();

        AddVertices(mesh);
        AddTriangles(mesh);
        AddUVs(mesh);
        //AddMaterials(mesh);

        mesh.RecalculateNormals();
        mesh.RecalculateBounds();
        mesh.Optimize();
        return mesh;
    }

    void AddVertices(Mesh mesh)
    {
        Vector3 p0 = new Vector3(0, 0, 0);
        Vector3 p1 = new Vector3(1, 0, 0);
        Vector3 p2 = new Vector3(0.5f, 0, Mathf.Sqrt(0.75f));
        Vector3 p3 = new Vector3(0.5f, Mathf.Sqrt(0.75f), Mathf.Sqrt(0.75f) / 3);

        mesh.vertices = new Vector3[]
            {
				p0,p1,p2,
				p0,p2,p3,
				p2,p1,p3,
				p0,p3,p1
			};
    }

    void AddTriangles(Mesh mesh)
    {
        mesh.triangles = new int[]
            {
				0,1,2,
				3,4,5,
				6,7,8,
				9,10,11
			};
    }

    void AddUVs(Mesh mesh)
    {
        Vector2 uv3a = new Vector2(0, 0);
        Vector2 uv1 = new Vector2(0.5f, 0);
        Vector2 uv0 = new Vector2(0.25f, Mathf.Sqrt(0.75f) / 2);
        Vector2 uv2 = new Vector2(0.75f, Mathf.Sqrt(0.75f) / 2);
        Vector2 uv3b = new Vector2(0.5f, Mathf.Sqrt(0.75f));
        Vector2 uv3c = new Vector2(1, 0);


        mesh.uv = new Vector2[]
            {
				uv0,uv1,uv2,
				uv0,uv2,uv3b,
				uv0,uv1,uv3a,
				uv1,uv2,uv3c
			};
    }

    void AddMaterials(MeshRenderer renderer)
    {
        renderer.material = new Material(Shader.Find("Diffuse"));
    }
}
